@@ -3,22 +3,19 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -45,19 +42,15 @@ enum {
 	kIVADInvertDataMask	= 0x100,
 };
 
-
-static void RestoreXPRAM( IODisplay * display, OSDictionary * params );
-static void SaveXPRAM( OSDictionary * params );
-
 static IOReturn
 ApplePMUSendMiscCommand( UInt32 command,
                         IOByteCount sendLength, UInt8 * sendBuffer,
                         IOByteCount * readLength, UInt8 * readBuffer );
 
-IOReturn
+static IOReturn
 AppleCudaWriteIIC( UInt8 address, const UInt8 * buffer, IOByteCount * count );
 
-IOReturn
+static IOReturn
 AppleCudaReadIIC( UInt8 address, UInt8 * buffer, IOByteCount * count );
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -176,25 +169,6 @@ private:
 	bool setupVCP( void );
 	};
 
-
-class AppleG3AIO : public IODisplayParameterHandler
-{
-    OSDeclareDefaultStructors(AppleG3AIO);
-
-private:
-    OSDictionary *	fDisplayParams;
-
-public:
-    virtual IOService * probe( IOService * provider, SInt32 * score );
-    virtual bool start(	IOService * provider );
-
-    virtual bool setDisplay( IODisplay * display );
-    virtual bool doIntegerSet( OSDictionary * params,
-                               const OSSymbol * paramName, UInt32 value );
-    virtual bool doDataSet( const OSSymbol * paramName, OSData * value );
-    virtual bool doUpdate( void );
-};
-
 class ApplePanel : public IODisplayParameterHandler
 {
     OSDeclareAbstractStructors(ApplePanel);
@@ -262,7 +236,6 @@ public:
 OSDefineMetaClassAndStructors(AppleIVAD, IODisplayParameterHandler);
 OSDefineMetaClassAndStructors(AppleIVAD2, IODisplayParameterHandler);
 OSDefineMetaClassAndStructors(AppleVCP, IODisplayParameterHandler);
-OSDefineMetaClassAndStructors(AppleG3AIO, IODisplayParameterHandler);
 
 OSDefineMetaClass(ApplePanel, IODisplayParameterHandler);
 OSDefineAbstractStructors(ApplePanel, IODisplayParameterHandler);
@@ -2317,20 +2290,6 @@ bool ApplePanel::start( IOService * provider )
 
     fBacklightNode = provider;
 
-    iter = getMatchingServices( nameMatching("display") );
-    if( iter) {
-        while( (next = OSDynamicCast(IOService, iter->getNextObject()))) {
-
-            while( next && !next->metaCast("IOPCIDevice"))
-                next = OSDynamicCast(IOService, next->getParentEntry( gIODTPlane ));
-            if( 0 == next)
-                continue;
-            
-            attach( next );
-            break;
-        }
-        iter->release();
-    }
     if( fBacklightNode
      && (data = OSDynamicCast( OSData, fBacklightNode->getProperty("panel-family"))))
         fPanelFamily = *((UInt32 *) data->getBytesNoCopy());
@@ -2515,6 +2474,8 @@ bool ApplePanelA::doIntegerSet( OSDictionary * params, const OSSymbol * paramNam
             continue;
 
         if( paramName == gIODisplayParametersCommitKey) {
+	    if (kBacklightPowerOff == info.backlightOnOff)
+		info.backlightLevel = fPanelInfo->off;
             info.saveNVRAM = 1;
         } else {
             info.saveNVRAM = 0;
@@ -2527,7 +2488,7 @@ bool ApplePanelA::doIntegerSet( OSDictionary * params, const OSSymbol * paramNam
 
         err = fFramebuffer->doControl( kPanelABacklightControl, &info);
         if( err)
-            continue;
+	    continue;
         fBacklightNode->setProperty("ApplePanelRawBrightness", info.backlightLevel, 32);
         ok = true;
 
@@ -2545,10 +2506,15 @@ bool ApplePanelA::doUpdate( void )
     bzero(&info, sizeof(info));
     err = fFramebuffer->doStatus( kPanelABacklightControl, &info);
     if( kIOReturnSuccess == err) {
-        UInt32 raw = info.backlightLevel;
-        for( value = kApplePanelLevels;
-             (value > 0) && ((raw < fValues[value]) || (raw == fValues[value - 1]));
-             value--)	{}
+	if (kBacklightPowerOff == info.backlightOnOff)
+	    value = 0;
+	else
+	{
+	    UInt32 raw = info.backlightLevel;
+	    for( value = kApplePanelLevels;
+		(value > 0) && ((raw < fValues[value]) || (raw == fValues[value - 1]));
+		value--)	{}
+	}
     }
     IODisplay::setParameter( fDisplayParams, gIODisplayBrightnessKey, value );
 
@@ -2708,124 +2674,6 @@ bool ApplePMUPanel::doUpdate( void )
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-#undef super
-#define super IODisplayParameterHandler
-
-#if 1
-#define 	heathrowBrightnessControl ((volatile UInt8 *)0xf3000033)
-#define 	heathrowContrastControl ((volatile UInt8 *)0xf3000032)
-#endif
-#if 0
-#define		defaultBrightness	144
-#define		defaultContrast		183
-#endif
-
-IOService * AppleG3AIO::probe( IOService * provider, SInt32 * score )
-{
-    if( 0 == getPlatform()->getProperty("AllInOne"))
-        return( 0 );
-    else
-        return( super::probe( provider, score ));
-}
-
-bool AppleG3AIO::start( IOService * provider )
-{
-    if( !super::start( provider))
-        return( false );
-
-    fDisplayParams = OSDynamicCast(OSDictionary, getProperty(gIODisplayParametersKey));
-    if( !fDisplayParams)
-        return( false );
-
-    return( true );
-}
-
-bool AppleG3AIO::setDisplay( IODisplay * display )
-{
-    OSNumber * num;
-
-    display->setProperty( gIODisplayParametersKey, fDisplayParams);
-
-    num = OSNumber::withNumber( kAppleOnboardGUID, 64 );
-    display->setProperty( gIODisplayGUIDKey, num);
-    num->release();
-
-    RestoreXPRAM( display, fDisplayParams );
-
-    return( true );
-}
-
-bool AppleG3AIO::doDataSet( const OSSymbol * paramName, OSData * value )
-{
-    return( false );
-}
-
-bool AppleG3AIO::doIntegerSet( OSDictionary * params, const OSSymbol * paramName, UInt32 value )
-{
-    if( paramName == gIODisplayBrightnessKey) {
-        *heathrowBrightnessControl = value;
-
-    } else if( paramName == gIODisplayContrastKey) {
-        *heathrowContrastControl = value;
-
-    } else if( paramName == gIODisplayParametersCommitKey)
-        SaveXPRAM( fDisplayParams );
-    else
-        return( false );
-
-    return( true );
-}
-
-bool AppleG3AIO::doUpdate( void )
-{
-    return( false );
-}
-
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-
-static void RestoreXPRAM( IODisplay * display, OSDictionary * params )
-{
-    UInt8 bytes[2];
-    SInt32 value, min, max;
-    OSDictionary * param;
-    IODTPlatformExpert * dtPlaform = OSDynamicCast( IODTPlatformExpert, IOService::getPlatform() );
-
-    if( !dtPlaform
-    || (kIOReturnSuccess != dtPlaform->readXPRAM( 0x50, (UInt8 *) bytes, 2 )))
-        bytes[0] = bytes[1] = 0;
-
-    if( (param = IODisplay::getIntegerRange( params, gIODisplayBrightnessKey, 0, &min, &max))) {
-        value = (bytes[0] + (((UInt8) max) + ((UInt8) min)) / 2) & 0xff;
-        display->setForKey( param, gIODisplayBrightnessKey, value, min, max);
-    }
-    if( (param = IODisplay::getIntegerRange( params, gIODisplayContrastKey, 0, &min, &max))) {
-        value = (bytes[1] + ((UInt8) max)) & 0xff;
-        display->setForKey( param, gIODisplayContrastKey, value, min, max);
-    }
-}
-
-
-static void SaveXPRAM( OSDictionary * params )
-{
-    UInt8 bytes[2];
-    SInt32 value, min, max;
-    OSDictionary * param;
-    IODTPlatformExpert * dtPlaform = OSDynamicCast( IODTPlatformExpert, IOService::getPlatform() );
-
-    if( dtPlaform
-    && (kIOReturnSuccess == dtPlaform->readXPRAM( 0x50, (UInt8 *) bytes, 2 ))) {
-
-        if( (param = IODisplay::getIntegerRange( params, gIODisplayBrightnessKey, &value, &min, &max)))
-            bytes[0] = value - (min + max) / 2;
-
-        if( (param = IODisplay::getIntegerRange( params, gIODisplayContrastKey, &value, &min, &max)))
-            bytes[1] = value - max;
-        dtPlaform->writeXPRAM( 0x50, (UInt8 *) bytes, 2 );
-    }
-}
-
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-
 static IOReturn
 ApplePMUSendMiscCommand( UInt32 command,
                         IOByteCount sendLength, UInt8 * sendBuffer,
@@ -2852,3 +2700,40 @@ ApplePMUSendMiscCommand( UInt32 command,
                                          (void*)&params, NULL, NULL, NULL );
     return( ret );
 }
+
+static IOReturn
+AppleCudaIIC( const OSSymbol * sym, UInt8 address, UInt8 * buffer, IOByteCount * count )
+{
+    static IOService * cuda;
+    IOReturn ret = kIOReturnError;
+
+    // Wait for Cuda to show up:
+    if (!cuda)
+        cuda = IOService::waitForService(IOService::serviceMatching("AppleCuda"));
+    if (cuda)
+        ret = cuda->callPlatformFunction( sym, false, (void *) (UInt32) address, buffer, count, 0 );
+
+    return (ret);
+}
+
+static IOReturn
+AppleCudaWriteIIC( UInt8 address, const UInt8 * buffer, IOByteCount * count )
+{
+    static const OSSymbol * sym;
+
+    if (!sym)
+        sym = OSSymbol::withCString("write_iic");
+
+    return (AppleCudaIIC(sym, address, (UInt8 *) buffer, count));
+}
+
+static IOReturn
+AppleCudaReadIIC( UInt8 address, UInt8 * buffer, IOByteCount * count )
+{
+    static const OSSymbol * sym;
+
+    if (!sym)
+        sym = OSSymbol::withCString("read_iic");
+
+    return (AppleCudaIIC(sym, address, buffer, count));
+}
